# authentic2 - versatile identity manager
# Copyright (C) 2010-2019 Entr'ouvert
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import logging
from django.utils import six


class RequestContextFilter(logging.Filter):
    DEFAULT_USERNAME = '-'
    DEFAULT_IP = '-'
    DEFAULT_REQUEST_ID = '-'

    def filter(self, record):
        '''Add username, ip and request ID to the log record.

           Inspired by django-log-request-id
        '''
        from . import middleware
        request = middleware.StoreRequestMiddleware.get_request()
        user = self.DEFAULT_USERNAME
        ip = self.DEFAULT_IP
        request_id = self.DEFAULT_REQUEST_ID
        if request is not None:
            if hasattr(request, 'user') and request.user.is_authenticated():
                user = six.text_type(request.user)
            ip = request.META.get('REMOTE_ADDR', self.DEFAULT_IP)
            request_id = request.request_id
        record.user = user
        record.ip = ip
        record.request_id = request_id
        return True


class ForceDebugFilter(logging.Filter):
    def filter(self, record):
        record.level = logging.DEBUG
        return True
